from blueness import module
from blue_options import string
from blue_objects.metadata import get_from_object
from blueflow.workflow.generic import Workflow

from palisades import NAME
from palisades.logger import logger

NAME = module.name(__file__, NAME)


def generate_ingest_workflow(
    job_name: str,
    query_object_name: str,
    datacube_ingest_options: str,
    predict_options: str,
    model_object_name: str,
    buildings_query_options: str,
    analysis_options: str,
    count: int = -1,
    do_tag: bool = True,
) -> bool:
    list_of_datacube_id = get_from_object(
        query_object_name,
        "datacube_id",
    )
    if count != -1:
        list_of_datacube_id = list_of_datacube_id[:count]

    logger.info(
        "{}.generate_ingest_workflow: {} @ {} {} {} {} {} -{}> {}".format(
            NAME,
            query_object_name,
            datacube_ingest_options,
            predict_options,
            model_object_name,
            buildings_query_options,
            analysis_options,
            "tag-" if do_tag else "",
            job_name,
        )
    )

    workflow = Workflow(
        job_name,
        name="palisades.ingest",
        args={
            "query_object_name": query_object_name,
            "datacube_ingest_options": datacube_ingest_options,
            "predict_options": predict_options,
            "model_object_name": model_object_name,
            "buildings_query_options": buildings_query_options,
            "analysis_options": analysis_options,
            "list_of_datacube_id": list_of_datacube_id,
            "do_tag": do_tag,
        },
    )

    suffix = string.pretty_date(
        as_filename=True,
        include_time=False,
        unique=True,
    )
    for index, datacube_id in enumerate(list_of_datacube_id):
        node = f"{index:04d}"

        workflow.G.add_node(node)

        prediction_object_name = f"predict-{datacube_id}-{suffix}"

        workflow.G.nodes[node]["command_line"] = " ".join(
            [
                "blueflow_workflow monitor",
                f"node={node}",
                job_name,
                "palisades_predict",
                f"tag={int(do_tag)}",
                datacube_ingest_options,
                predict_options,
                model_object_name,
                datacube_id,
                prediction_object_name,
                buildings_query_options,
                analysis_options,
            ]
        )

    return workflow.save()



from typing import List

from abcli.help.generic import help_functions as generic_help_functions
from blue_options.terminal import show_usage, xtra
from roofai.help.semseg import (
    train_options,
    device_and_profile_details,
    predict_options,
)
from blueflow.help.workflow import submit_options as workflow_submit_options
from blueflow.help.workflow import runner_details
from blue_geo.watch.targets.target_list import TargetList
from blue_geo.help.datacube import scope_details
from blue_geo.help.datacube import ingest_options as datacube_ingest_options
from blue_geo.help.datacube.label import options as datacube_label_options

from palisades.help.buildings import help_functions as help_buildings
from palisades.help.buildings import query_options as building_query_options
from palisades.help.buildings import query_details as building_query_details
from palisades.help.buildings import analyze_options as building_analyze_options
from palisades.help.buildings import analyze_details as building_analyze_details
from palisades.help.analytics import help_functions as help_analytics
from palisades import ALIAS

target_list = TargetList(catalog="maxar_open_data")


def building_query_options_(mono: bool):
    return "".join(
        [
            xtra("~download_footprints | ", mono=mono),
            building_query_options(mono=mono),
        ]
    )


def building_analyze_options_(mono: bool):
    return "".join(
        [
            xtra("~analyze | ", mono=mono),
            building_analyze_options(
                mono=mono,
                cascade=True,
            ),
        ]
    )


def datacube_ingest_options_(mono: bool):
    return "".join(
        [
            xtra("~ingest | ", mono=mono),
            datacube_ingest_options(mono=mono),
        ]
    )


def help_ingest(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("~download,dryrun", mono=mono)

    target_options = "".join(
        [
            "target=<target>",
            xtra(" | <query-object-name>", mono),
        ]
    )

    batch_options = "".join(
        [
            "predict",
            xtra(",count=<count>,~tag", mono=mono),
        ]
    )

    workflow_options = "".join(
        [
            xtra("~submit | ", mono=mono),
            workflow_submit_options(
                mono=mono,
                cascade=True,
            ),
        ]
    )

    return show_usage(
        [
            "palisades",
            "ingest",
            f"[{options}]",
            f"[{target_options}]",
            f"[{datacube_ingest_options_(mono=mono)}]",
            f"[{batch_options}]",
            "[{}]".format(predict_options(mono=mono, cascade=True)),
            xtra("[-|<model-object-name>]", mono=mono),
            f"[{building_query_options_(mono=mono)}]",
            f"[{building_analyze_options_(mono=mono)}]",
            f"[{workflow_options}]",
        ],
        "ingest <target>.",
        {
            "target: {}".format(" | ".join(target_list.get_list())): [],
            **scope_details,
            **device_and_profile_details,
            **building_query_details,
            **building_analyze_details,
            **runner_details,
        },
        mono=mono,
    )


def help_label(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "download,offset=<offset>"

    return show_usage(
        [
            "palisades",
            "label",
            f"[{options}]",
            f"[{datacube_label_options(mono=mono)}]",
            "[.|<query-object-name>]",
        ],
        "label <query-object-name>.",
        mono=mono,
    )


def help_predict(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("~tag", mono=mono)

    return show_usage(
        [
            "palisades",
            "predict",
            f"[{options}]",
            f"[{datacube_ingest_options_(mono=mono)}]",
            "[{}]".format(predict_options(mono=mono, cascade=True)),
            xtra("[-|<model-object-name>]", mono=mono),
            "[.|<datacube-id>]",
            "[-|<prediction-object-name>]",
            f"[{building_query_options_(mono=mono)}]",
            f"[{building_analyze_options_(mono=mono)}]",
        ],
        "<datacube-id> -<model-object-name>-> <prediction-object-name>",
        {
            **device_and_profile_details,
            **building_query_details,
            **building_analyze_details,
        },
        mono=mono,
    )


def help_train(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun,~download,review", mono=mono)

    ingest_options = "".join(
        [
            "count=<10000>",
            xtra(",dryrun,upload", mono=mono),
        ]
    )

    return show_usage(
        [
            "palisades",
            "train",
            f"[{options}]",
            "[.|<query-object-name>]",
            f"[{ingest_options}]",
            "[-|<dataset-object-name>]",
            "[{},epochs=<5>]".format(
                train_options(
                    mono=mono,
                    show_download=False,
                )
            ),
            "[-|<model-object-name>]",
        ],
        "train palisades.",
        device_and_profile_details,
        mono=mono,
    )


help_functions = generic_help_functions(plugin_name=ALIAS)

help_functions.update(
    {
        "buildings": help_buildings,
        "ingest": help_ingest,
        "analytics": help_analytics,
        "label": help_label,
        "predict": help_predict,
        "train": help_train,
    }
)



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from palisades import NAME
from palisades.analytics.ingest import ingest_analytics
from palisades.analytics.render import render_analytics
from palisades.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="ingest | render",
)
parser.add_argument(
    "--object_name",
    type=str,
)
parser.add_argument(
    "--acq_count",
    type=int,
    default=-1,
    help="-1: all",
)
parser.add_argument(
    "--building_count",
    type=int,
    default=-1,
    help="-1: all",
)
parser.add_argument(
    "--building_id",
    type=str,
)
parser.add_argument(
    "--verbose",
    type=int,
    default=0,
    help="0 | 1",
)
args = parser.parse_args()

success = False
if args.task == "ingest":
    success = ingest_analytics(
        object_name=args.object_name,
        acq_count=args.acq_count,
        building_count=args.building_count,
        verbose=args.verbose == 1,
    )
elif args.task == "render":
    success = render_analytics(
        object_name=args.object_name,
        building_id=args.building_id,
        verbose=args.verbose == 1,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import os
from blue_options.env import load_config, load_env

load_env(__name__)
load_config(__name__)


PALISADES_SECRET = os.getenv(
    "PALISADES_SECRET",
    "",
)

PALISADES_QUERY_OBJECT_PALISADES_MAXAR = os.getenv(
    "PALISADES_QUERY_OBJECT_PALISADES_MAXAR",
    "",
)

PALISADES_QUERY_OBJECT_PALISADES_MAXAR_TEST = os.getenv(
    "PALISADES_QUERY_OBJECT_PALISADES_MAXAR_TEST",
    "",
)

PALISADES_DEFAULT_FIRE_MODEL = os.getenv(
    "PALISADES_DEFAULT_FIRE_MODEL",
    "",
)

PALISADES_QGIS_TEMPLATE_PREDICT = os.getenv(
    "PALISADES_QGIS_TEMPLATE_PREDICT",
    "",
)

PALISADES_QGIS_TEMPLATE_ANALYTICS = os.getenv(
    "PALISADES_QGIS_TEMPLATE_ANALYTICS",
    "",
)

PALISADES_TEST_PREDICTION_OBJECT = os.getenv(
    "PALISADES_TEST_PREDICTION_OBJECT",
    "",
)

PALISADES_TEST_DATACUBE = os.getenv(
    "PALISADES_TEST_DATACUBE",
    "",
)

PALISADES_DEFAULT_BUFFER_M_str = os.getenv(
    "PALISADES_DEFAULT_BUFFER_M",
    "",
)
try:
    PALISADES_DEFAULT_BUFFER_M = float(PALISADES_DEFAULT_BUFFER_M_str)
except Exception:
    PALISADES_DEFAULT_BUFFER_M = 10

PALISADES_DAMAGE_THRESHOLD_str = os.getenv("PALISADES_DAMAGE_THRESHOLD", "")
try:
    PALISADES_DAMAGE_THRESHOLD = float(PALISADES_DAMAGE_THRESHOLD_str)
except Exception:
    PALISADES_DAMAGE_THRESHOLD = 0.1



import os
import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from palisades import NAME
from palisades.workflow.ingest import generate_ingest_workflow
from palisades.logger import logger

NAME = module.name(__file__, NAME)

list_of_tasks = "generate"


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help=list_of_tasks,
)
parser.add_argument(
    "--workflow_name",
    type=str,
)
parser.add_argument(
    "--job_name",
    type=str,
)
parser.add_argument(
    "--query_object_name",
    type=str,
)
parser.add_argument(
    "--count",
    type=int,
)
parser.add_argument(
    "--do_tag",
    type=int,
)
parser.add_argument(
    "--datacube_ingest_options",
    type=str,
)
parser.add_argument(
    "--predict_options",
    type=str,
)
parser.add_argument(
    "--model_object_name",
    type=str,
)
parser.add_argument(
    "--buildings_query_options",
    type=str,
)
parser.add_argument(
    "--analysis_options",
    type=str,
)
args = parser.parse_args()

success = args.task in list_of_tasks
if args.task == "generate":
    if args.workflow_name == "ingest":
        success = generate_ingest_workflow(
            job_name=args.job_name,
            query_object_name=args.query_object_name,
            count=args.count,
            do_tag=args.do_tag == 1,
            datacube_ingest_options=args.datacube_ingest_options,
            predict_options=args.predict_options,
            model_object_name=args.model_object_name,
            buildings_query_options=args.buildings_query_options,
            analysis_options=args.analysis_options,
        )
    else:
        success = False
        logger.error("{}: workflow not found.")
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import Dict, Any
from tqdm import tqdm, trange
import geopandas as gpd
import numpy as np
import matplotlib.pyplot as plt

from blueness import module
from blue_objects import mlflow, objects, file
from blue_objects.metadata import post_to_object, get_from_object
from blue_objects.mlflow.tags import create_filter_string
from blue_objects.graphics.signature import sign_filename

from palisades import NAME
from palisades import env
from palisades.host import signature
from palisades.logger import logger

NAME = module.name(__file__, NAME)


def ingest_analytics(
    object_name: str,
    acq_count: int = -1,
    building_count: int = -1,
    verbose: bool = False,
    damage_threshold: float = env.PALISADES_DAMAGE_THRESHOLD,
) -> bool:
    logger.info(
        "{}.ingest_analytics -{}{}> {}".format(
            NAME,
            f"{acq_count} acq(s)-" if acq_count != -1 else "",
            f"{building_count} buildings(s)-" if building_count != -1 else "",
            object_name,
        )
    )

    list_of_prediction_objects = mlflow.search(
        create_filter_string("contains=palisades.prediction,profile=FULL")
    )
    if acq_count != -1:
        list_of_prediction_objects = list_of_prediction_objects[:acq_count]
    logger.info(f"{len(list_of_prediction_objects)} acq(s) to process.")

    object_metadata: Dict[str, Any] = {}
    list_of_buildings: Dict[str, Any] = {}
    success_count: int = 0
    list_of_polygons = []
    list_of_building_ids = []
    list_of_thumbnail = []
    list_of_thumbnail_object = []
    list_of_prediction_datetime = []
    crs = ""
    for prediction_object_name in tqdm(list_of_prediction_objects):
        logger.info(f"processing {prediction_object_name} ...")

        object_metadata[prediction_object_name] = {"success": False}

        prediction_datetime = get_from_object(
            prediction_object_name,
            "analysis.datetime",
            download=True,
        )
        if not prediction_datetime:
            logger.warning("analysis.datetime not found.")
            continue
        list_of_prediction_datetime += [prediction_datetime]

        if not objects.download(
            filename="analysis.gpkg",
            object_name=prediction_object_name,
        ):
            continue

        success, gdf = file.load_geodataframe(
            objects.path_of(
                filename="analysis.gpkg",
                object_name=prediction_object_name,
            ),
            log=verbose,
        )
        if not success:
            continue
        if not crs:
            crs = gdf.crs
        if building_count != -1:
            gdf = gdf.head(building_count)

        if "building_id" not in gdf.columns:
            logger.warning("building_id not found.")
            continue

        for _, row in tqdm(gdf.iterrows()):
            if row["building_id"] not in list_of_building_ids:
                list_of_buildings[row["building_id"]] = {}

                list_of_building_ids.append(row["building_id"])
                list_of_polygons.append(row["geometry"])
                list_of_thumbnail.append(row["thumbnail"])
                list_of_thumbnail_object.append(prediction_object_name)

            list_of_buildings[row["building_id"]][prediction_datetime] = {
                "area": row["area"],
                "damage": row["damage"],
                "thumbnail": row["thumbnail"],
                "object_name": prediction_object_name,
            }

        object_metadata[prediction_object_name] = {
            "success": True,
            "building_count": len(gdf),
        }
        success_count += 1

    list_of_prediction_datetime = sorted(list_of_prediction_datetime)

    observation_count: Dict[int:int] = {}
    for building_metadata in list_of_buildings.values():
        observation_count[len(building_metadata)] = (
            observation_count.get(len(building_metadata), 0) + 1
        )
    logger.info(
        "observation counts: {}".format(
            ", ".join(
                [f"{rounds}X: {count}" for rounds, count in observation_count.items()]
            )
        )
    )

    output_gdf = gpd.GeoDataFrame(
        data={
            "building_id": list_of_building_ids,
            "geometry": list_of_polygons,
            "area": [
                float(
                    np.mean(
                        [
                            building_info["area"]
                            for building_info in building_metadata.values()
                        ]
                    )
                )
                for building_metadata in list_of_buildings.values()
            ],
            "area_std": [
                float(
                    np.std(
                        [
                            building_info["area"]
                            for building_info in building_metadata.values()
                        ]
                    )
                )
                for building_metadata in list_of_buildings.values()
            ],
            "damage": [
                float(
                    np.mean(
                        [
                            building_info["damage"]
                            for building_info in building_metadata.values()
                        ]
                    )
                )
                for building_metadata in list_of_buildings.values()
            ],
            "damage_std": [
                float(
                    np.std(
                        [
                            building_info["damage"]
                            for building_info in building_metadata.values()
                        ]
                    )
                )
                for building_metadata in list_of_buildings.values()
            ],
            "thumbnail": list_of_thumbnail,
            "thumbnail_object": list_of_thumbnail_object,
            "observation_count": [
                len(building_metadata)
                for building_metadata in list_of_buildings.values()
            ],
        },
    )
    output_gdf.crs = crs
    geojson_filename = objects.path_of(
        "analytics.geojson",
        object_name,
    )
    output_gdf.to_file(
        geojson_filename,
        driver="GeoJSON",
    )

    logger.info(
        "{} object(s) -> {} ingested -> {:,} buildings(s) -> {}.".format(
            len(object_metadata),
            success_count,
            len(output_gdf),
            file.name_and_extension(geojson_filename),
        )
    )

    list_of_building_states = {
        "green": "zero damage",
        "yellow": "some damage",
        "red": "heavy damage",
    }

    prediction_datetime_summary: Dict[str, Dict[str, int]] = {
        prediction_datetime: {
            building_state: 0 for building_state in list_of_building_states
        }
        for prediction_datetime in list_of_prediction_datetime
    }
    for building_metadata in list_of_buildings.values():
        for prediction_datetime, building_info in building_metadata.items():
            building_state = (
                "green"
                if building_info["damage"] == 0
                else "yellow" if building_info["damage"] <= damage_threshold else "red"
            )

            prediction_datetime_summary[prediction_datetime][building_state] += 1

    plt.figure(figsize=(10, 5))
    bin_centers = np.arange(len(list_of_prediction_datetime))
    bar_width = 0.2
    for index_, building_state in enumerate(list_of_building_states):
        plt.bar(
            bin_centers + index_ * bar_width,
            [
                prediction_datetime_summary[prediction_datetime][building_state]
                for prediction_datetime in list_of_prediction_datetime
            ],
            width=bar_width,
            label=list_of_building_states[building_state],
            align="center",
            color=building_state,
            alpha=0.5,
        )
    plt.title("Damage History")
    plt.xlabel("Acquisition Date")
    plt.ylabel("Building Count")
    plt.xticks(
        bin_centers,
        list_of_prediction_datetime,
        rotation=90,
    )
    plt.grid(True)
    plt.legend()
    filename = objects.path_of(
        "damage-history.png",
        object_name,
    )
    if not file.save_fig(filename):
        return False
    if not sign_filename(
        filename=filename,
        header=[
            "{} acquisition(s)".format(
                len(
                    list_of_prediction_datetime,
                )
            ),
        ]
        + objects.signature(object_name=object_name),
        footer=signature(),
    ):
        return False

    return post_to_object(
        object_name,
        "analytics",
        {
            "list_of_buildings": list_of_buildings,
            "list_of_prediction_datetime": list_of_prediction_datetime,
            "building_count": len(list_of_buildings),
            "objects": object_metadata,
            "observation_count": observation_count,
            "prediction_datetime_summary": prediction_datetime_summary,
        },
    )



from typing import List

from blue_options.terminal import show_usage, xtra


def help_ingest(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            xtra("acq=<-1>,buildings=<-1>,dryrun,", mono=mono),
            "upload",
        ]
    )

    return show_usage(
        [
            "palisades",
            "analytics",
            "ingest",
            f"[{options}]",
            "[-|<object-name>]",
        ],
        "ingest analytics.",
        mono=mono,
    )


def help_render(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "building=<building-id>",
            xtra(",~download,dryrun,", mono=mono),
            "upload",
        ]
    )

    return show_usage(
        [
            "palisades",
            "analytics",
            "render",
            f"[{options}]",
            "[.|<object-name>]",
        ],
        "render analytics.",
        mono=mono,
    )


help_functions = {
    "ingest": help_ingest,
    "render": help_render,
}



import geopandas as gpd
from tqdm import tqdm

from blueness import module
from blue_objects import objects, file
from blue_objects.metadata import get_from_object
from blue_objects.graphics.gif import generate_animated_gif

from palisades import NAME
from palisades.logger import logger
from typing import List

NAME = module.name(__file__, NAME)


def render_analytics(
    object_name: str,
    building_id: str,
    log: bool = True,
    verbose: bool = False,
) -> bool:
    logger.info(
        "{}.render_analytics: {} @ {}".format(
            NAME,
            building_id,
            object_name,
        )
    )

    geojson_filename = objects.path_of(
        "analytics.geojson",
        object_name,
    )
    success, gdf = file.load_geodataframe(
        geojson_filename,
        log=log,
    )
    if not success:
        return success

    list_of_buildings = get_from_object(
        object_name,
        "analytics.list_of_buildings",
        [],
    )
    if building_id not in list_of_buildings:
        logger.error(f"{building_id}: building-id not found.")
        return False
    building_metadata = list_of_buildings[building_id]

    list_of_images: List[str] = []
    for prediction_info in tqdm(building_metadata.values()):
        if not objects.download(
            filename=prediction_info["thumbnail"],
            object_name=prediction_info["object_name"],
        ):
            return False

        list_of_images += [
            objects.path_of(
                filename=prediction_info["thumbnail"],
                object_name=prediction_info["object_name"],
            )
        ]

    thumbnail_filename = f"thumbnail-{building_id}-{object_name}.gif"

    if not generate_animated_gif(
        list_of_images=list_of_images,
        output_filename=objects.path_of(
            thumbnail_filename,
            object_name,
        ),
        frame_duration=1000,
        log=log,
    ):
        return False

    gdf.loc[gdf["building_id"] == building_id, "thumbnail"] = thumbnail_filename
    gdf.loc[gdf["building_id"] == building_id, "thumbnail_object"] = object_name

    return file.save_geojson(
        geojson_filename,
        gdf,
        log=log,
    )
